/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
#include "nsISupports.idl"

interface nsIDocShell;
interface mozIDOMWindowProxy;

/**
 * XXX This interface seems to be unimplemented and unused.
 */
[scriptable, uuid(8eb6ffc9-715c-487e-927c-c56139229548)]
interface nsIPrintStatusFeedback : nsISupports {
	void showStatusString(in wstring status);
	void startMeteors();
	void stopMeteors();
	void showProgress(in long percent);
	[noscript] void setDocShell(in nsIDocShell shell, in mozIDOMWindowProxy window);
	void closeWindow();
};

