#define MOZ_SOURCE_STAMP 0723a0212f5e0b30d7532d4e35eba7759fb54507
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/0723a0212f5e0b30d7532d4e35eba7759fb54507
